package io.github.null2264.tsukumogami.core.commands

import io.github.null2264.tsukumogami.core.Context
import io.github.null2264.tsukumogami.core.module.BotModule
import kotlin.reflect.KFunction

open class Command(
    val name: String,
    val alias: Set<String>,
    val description: String,
    private val arguments: KFunction<Arguments>,
    private val handler: suspend (Context, Arguments) -> Unit,
) {

    /**
     * Return parent command if this command is a subcommand otherwise it returns null
     */
    var parent: Command? = null

    /**
     * The module this command belong to
     */
    var module: BotModule? = null
        get() {
            if (parent != null) {
                field = parent?.module
            }
            return field
        }

    open suspend fun invoke(context: Context) {
        val parsedArguments = arguments.call()
        parsedArguments.parse(context)
        handler.invoke(context, parsedArguments)
    }
}
