package io.github.null2264.tsukumogami.core.commands

import io.github.null2264.tsukumogami.core.Context
import io.github.null2264.tsukumogami.core.annotation.TsukumogamiInternalApi
import kotlin.reflect.KFunction

@OptIn(TsukumogamiInternalApi::class)
interface IGroup {

    val allCommands: MutableMap<String, Command>

    @TsukumogamiInternalApi
    fun _addCommand(command: Command) {
        allCommands[command.name] = command
        command.alias.forEach { allCommands[it] = command }
    }

    fun commands(
        name: String,
        alias: Set<String> = setOf(),
        description: String = "",
        handler: suspend (Context) -> Unit,
    ) {
        val command = Command(name, alias, description, ::EmptyArguments) { ctx, _ -> handler(ctx) }
        _addCommand(command)
    }

    fun <Args : Arguments> commands(
        name: String,
        alias: Set<String> = setOf(),
        description: String = "",
        arguments: KFunction<Args>,
        handler: suspend (Context, Args) -> Unit,
    ) {
        val command = Command(name, alias, description, arguments) { ctx, args -> handler(ctx, args as Args) }
        _addCommand(command)
    }

    fun groups(
        name: String,
        alias: Set<String> = setOf(),
        description: String = "",
        declaration: IGroup.() -> Unit,
    ) {
        val group = Group(name, alias, description)
        declaration(group)
        _addCommand(group)
    }
}
